////////////////////////////////////////////////////////////////////////////////
//
// Filename: 	uartdump.cpp
//
// Project:	wbuart32, a full featured UART with simulator
//
// Purpose:	To produce whatever output the UART produces.
//
// Creator:	Dan Gisselquist, Ph.D.
//		Gisselquist Technology, LLC
//
////////////////////////////////////////////////////////////////////////////////
//
// Copyright (C) 2017, Gisselquist Technology, LLC
//
// This program is free software (firmware): you can redistribute it and/or
// modify it under the terms of  the GNU General Public License as published
// by the Free Software Foundation, either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTIBILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program.  (It's in the $(ROOT)/doc directory, run make with no
// target there if the PDF file isn't present.)  If not, see
// <http://www.gnu.org/licenses/> for a copy.
//
// License:	GPL, v3, as defined and found on www.gnu.org,
//		http://www.gnu.org/licenses/gpl.html
//
//
////////////////////////////////////////////////////////////////////////////////
//
//
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <signal.h>
#include "verilated.h"
#include "verilated_vcd_c.h"
#include "Vtransmitter.h"
#include "uartsim.h"

int	main(int argc, char **argv) {
	Verilated::commandArgs(argc, argv);
	Vtransmitter	tb;
	UARTSIM		*uart;
	int		port = 0;
	unsigned	setup = 10415, clocks = 0, baudclocks;

	// tb.i_setup = setup;
	uart = new UARTSIM(port);
	uart->setup(setup);
	baudclocks = setup & 0xfffffff;

#define	VCDTRACE
#ifdef	VCDTRACE
	Verilated::traceEverOn(true);
	VerilatedVcdC* tfp = new VerilatedVcdC;
	tb.trace(tfp, 99);
	tfp->open("transmitdump.vcd");
#define	TRACE_POSEDGE	tfp->dump(10*clocks)
#define	TRACE_NEGEDGE	tfp->dump(10*clocks+5)
#define	TRACE_CLOSE	tfp->close()
#else
#define	TRACE_POSEDGE
#define	TRACE_NEGEDGE
#define	TRACE_CLOSE
#endif

	clocks = 0;
	while(clocks < 16*32*baudclocks) {

		if (clocks == 20)
			tb.transmit = 1;
		else
			tb.transmit = 0;

		tb.i_clk = 1;
		tb.eval();
		TRACE_POSEDGE;
		tb.i_clk = 0;
		tb.eval();
		TRACE_NEGEDGE;

		(*uart)(tb.TxD);
		clocks++;
	}

	TRACE_CLOSE;
	printf("\n\nSimulation complete\n");
}


// This program can be built with:
// g++ -Iobj_dir -I/usr/share/verilator/include uartdump.cpp /usr/share/verilator/include/verilated.cpp /usr/share/verilator/include/verilated_vcd_c.cpp uartsim.cpp obj_dir/Vtransmitter__ALL.a -o uartdump
//
